(**************************************************************************
* BlockCAD, a program for building with Lego like blocks on the computer
* Copyright (C) 1998  Anders Isaksson
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
* -------------------------------------------------------------------------
* I hope You send me a copy of any important changes/additions You do.
*
* E-mail: isaksson.etuna@ebox.tninet.se
*
**************************************************************************)

unit Advanced;

interface

uses
  Windows, StdCtrls, ExtCtrls, Controls, Classes, Forms, Graphics,
  RXCtrls, RXSpin, LCadPc2
  ;

type
  TAdvOptions = class(TForm)
    GroupBox1: TGroupBox;
    spStud: TRxSpeedButton;
    spTurn: TRxSpeedButton;
    LabX: TLabel;
    LabY: TLabel;
    LabZ: TLabel;
    spTransp: TRxSpeedButton;
    lx: TLabel;
    ly: TLabel;
    lz: TLabel;
    sx: TRxSpinButton;
    sy: TRxSpinButton;
    sz: TRxSpinButton;
    GroupBox2: TGroupBox;
    spReplaceColor: TRxSpeedButton;
    Bevel1: TBevel;
    sbColor: TRxSpeedButton;
    spUseThisColor: TRxSpeedButton;
    spSetIdentity: TRxSpeedButton;
    edNewID: TEdit;
    Label1: TLabel;
    spEditLDrawInfo: TRxSpeedButton;
    spClose: TRxSpeedButton;
    GroupBox3: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    spYoffsetL: TRxSpinEdit;
    spZoffsetL: TRxSpinEdit;
    spRotOffset: TRxSpinEdit;
    spUpdateLDraw: TRxSpeedButton;
    RxSpeedButton1: TRxSpeedButton;
    spStep: TRxSpinEdit;
    Label6: TLabel;
    spSetStep: TRxSpeedButton;
    edStepNo: TEdit;
    Label7: TLabel;
    spXoffsetL: TRxSpinEdit;
    spMatchLDRAWcolor: TRxSpeedButton;
    procedure spStudClick(Sender: TObject);
    procedure spTurnClick(Sender: TObject);
    procedure spColorClick(Sender: TObject);
    procedure spTranspClick(Sender: TObject);
    procedure spCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure sxBottomClick(Sender: TObject);
    procedure sxTopClick(Sender: TObject);
    procedure syBottomClick(Sender: TObject);
    procedure syTopClick(Sender: TObject);
    procedure szBottomClick(Sender: TObject);
    procedure szTopClick(Sender: TObject);
    procedure spReplaceColorClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure spUseThisColorClick(Sender: TObject);
    procedure spSetIdentityClick(Sender: TObject);
    procedure spEditLDrawInfoClick(Sender: TObject);
    procedure edNewIDChange(Sender: TObject);
    procedure spYoffsetLChange(Sender: TObject);
    procedure spZoffsetLChange(Sender: TObject);
    procedure spRotOffsetChange(Sender: TObject);
    procedure spUpdateLDrawClick(Sender: TObject);
    procedure RxSpeedButton1Click(Sender: TObject);
    procedure spStepChange(Sender: TObject);
    procedure spSetStepClick(Sender: TObject);
    procedure spXoffsetLChange(Sender: TObject);
    procedure spMatchLDRAWcolorClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ShowCoord(b: TBlock);
    procedure SelectionChanged;
  end;

var
  AdvOptions        : TAdvOptions;

implementation

uses
  SysUtils,
  main;

{$R *.DFM}

type
  TIterProc = procedure(b: TBlock);

var
  minx, miny, minz  : integer;
  LastSel           : integer;
  SelCnt            : integer;
  CurRec            : TLDrawRecord;
  CurInx            : integer;

procedure IterateSelection(Proc: TIterProc);
var
  i                 : integer;
begin
  minx := 99999;
  miny := 99999;
  minz := 99999;
  SelCnt := 0;
  for i := 0 to Pred(BlockList.Count) do begin
    if Assigned(BlockList[i]) and TBlock(BlockList[i]).Selected then begin
      Inc(SelCnt);
      LastSel := i;
      if Assigned(Proc) then
        Proc(TBlock(BlockList[i]));
      with TBlock(BlockList[i]) {, Pnt} do begin
        minx := Min(px, minx);
        miny := Min(py, miny);
        minz := Min(pz, minz);
      end;
    end;
  end
end;

procedure TAdvOptions.SelectionChanged;
begin
  LastSel := -1;
  IterateSelection(nil);
  if (LastSel >= 0) then begin
    if (SelCnt = 1) then begin
      spSetIdentity.Enabled := True;
      edNewId.Enabled := True;
      edNewId.Text := TBlock(BlockList[LastSel]).PartID;
      spSetStep.Enabled := True;
      edStepNo.Enabled := True;
      edStepNo.Text := IntToStr(TBlock(BlockList[LastSel]).Step);
    end
    else begin
      spSetIdentity.Enabled := True;
      edNewId.Enabled := True;
      edNewId.Text := TBlock(BlockList[LastSel]).PartID;
      spSetStep.Enabled := True;
      edStepNo.Enabled := True;
      edStepNo.Text := IntToStr(TBlock(BlockList[LastSel]).Step);
      //      spSetIdentity.Enabled   := False;
      //      edNewId.Enabled         := False;
    end
  end
  else begin
    spSetIdentity.Enabled := False;
    edNewId.Enabled := False;
    spSetStep.Enabled := False;
    edStepNo.Enabled := False;
  end
end;

procedure TAdvOptions.ShowCoord(b: TBlock);
begin
  if not Assigned(b) then begin
    lx.Caption := '0';
    ly.Caption := '0';
    lz.Caption := '0';
  end
  else begin
    with b {.Pnt} do begin
      lx.Caption := IntToStr(px);
      ly.Caption := IntToStr(py);
      lz.Caption := IntToStr(pz);
    end;
    edNewId.Text := b.PartId;
  end
end;

procedure TAdvOptions.FormCreate(Sender: TObject);
begin
  Top := 52;
  Left := 0;
  Width := GroupBox3.Left;
end;

procedure DoStuds(b: TBlock);
begin
  b.st := not b.st;
end;

procedure TAdvOptions.spStudClick(Sender: TObject);
begin
  IterateSelection(DoStuds);
  MainForm.RedrawPicture(nil);
  MainForm.SetFocus;
end;

procedure DoTurn(b: TBlock);
begin
  b.Turn;
end;

procedure TAdvOptions.spTurnClick(Sender: TObject);
begin
  IterateSelection(DoTurn);
  MainForm.RedrawPicture(nil);
  MainForm.SetFocus;
end;

procedure DoPartid(b: TBlock);
begin
  b.PartID := AdvOptions.edNewID.Text;
end;

procedure TAdvOptions.spSetIdentityClick(Sender: TObject);
begin
  IterateSelection(DoPartID);
end;

procedure DoStepNo(b: TBlock);
begin
  b.Step := StrToInt(AdvOptions.edStepNo.Text);
end;

procedure TAdvOptions.spSetStepClick(Sender: TObject);
begin
  IterateSelection(DoStepNo);
end;

procedure DoColor(b: TBlock);
begin
  b.cl := MainForm.ColorIndicator;
end;

procedure TAdvOptions.spColorClick(Sender: TObject);
begin
  IterateSelection(DoColor);
  MainForm.RedrawPicture(nil);
  MainForm.SetFocus;
end;

procedure DoTransp(b: TBlock);
begin
  b.Transp := not b.Transp;
end;

procedure TAdvOptions.spTranspClick(Sender: TObject);
begin
  IterateSelection(DoTransp);
  MainForm.RedrawPicture(nil);
  MainForm.SetFocus;
end;

procedure TAdvOptions.spReplaceColorClick(Sender: TObject);
var
  i                 : integer;
  cl                : TColor;
begin
  cl := -1;
  for i := 0 to Pred(BlockList.Count) do begin
    if TBlock(BlockList[i]).Selected then begin
      cl := TBlock(BlockList[i]).cl;
      break;
    end
  end;
  if cl <> -1 then begin
    for i := 0 to Pred(BlockList.Count) do begin
      if TBlock(BlockList.Items[i]).cl = cl then
        TBlock(BlockList.Items[i]).cl := MainForm.ColorIndicator;
    end;
    MainForm.RedrawPicture(nil);
    MainForm.SetFocus;
  end
end;

procedure TAdvOptions.spMatchLDRAWcolorClick(Sender: TObject);
  function NearestLDRAWColor(c: TColor): TColor;
  var
    i               : integer;
    d               : TMyFloat;
    t               : TMyFloat;
    R, G, B         : integer;
    dx              : integer;
  begin
    c := ColorToRGB(c);
    dx := 15;
    d := Sqr(255) * 3 + 1;
    B := (integer(c) and $FF0000) shr 16;
    G := (integer(c) and $FF00) shr 8;
    R := (integer(c) and $FF);
    for i := 0 to High(MatchColor) do begin
      t := Sqr(R - MatchColor[i].R) + Sqr(G - MatchColor[i].G) + Sqr(B - MatchColor[i].B);
      if (t < d) then begin
        d := t;
        dx := i;
      end
    end;
    Result := RGB(MatchColor[dx].R, MatchColor[dx].G, MatchColor[dx].B);
  end;
var
  i                 : integer;
begin
  for i := 0 to Pred(BlockList.Count) do begin
    TBlock(BlockList.Items[i]).cl := NearestLDRAWColor(TBlock(BlockList.Items[i]).cl);
  end;
  MainForm.RedrawPicture(nil);
  MainForm.SetFocus;
end;

procedure TAdvOptions.spUseThisColorClick(Sender: TObject);
var
  i                 : integer;
begin
  for i := 0 to Pred(BlockList.Count) do begin
    if TBlock(BlockList[i]).Selected then begin
      MainForm.ColorIndicator := TBlock(BlockList[i]).cl;
      MainForm.RefreshListBox;
      break;
    end
  end
end;

procedure TAdvOptions.spCloseClick(Sender: TObject);
begin
  Hide;
end;

procedure DoXdown(b: TBlock);
begin
  with b do begin
    Dec(px, MainForm.ResXYFact);
  end
end;

procedure TAdvOptions.sxBottomClick(Sender: TObject);
begin
  IterateSelection(DoXdown);
  lx.Caption := IntToStr(minx);
  ly.Caption := IntToStr(miny);
  lz.Caption := IntToStr(minz);
  MainForm.RedrawPicture(nil);
  MainForm.SetFocus;
end;

procedure DoXup(b: TBlock);
begin
  with b do begin
    Inc(px, MainForm.ResXYFact);
  end
end;

procedure TAdvOptions.sxTopClick(Sender: TObject);
begin
  IterateSelection(DoXup);
  lx.Caption := IntToStr(minx);
  ly.Caption := IntToStr(miny);
  lz.Caption := IntToStr(minz);
  MainForm.RedrawPicture(nil);
  MainForm.SetFocus;
end;

procedure DoYdown(b: TBlock);
begin
  with b do begin
    Dec(py, MainForm.ResXYFact);
  end
end;

procedure TAdvOptions.syBottomClick(Sender: TObject);
begin
  IterateSelection(DoYdown);
  lx.Caption := IntToStr(minx);
  ly.Caption := IntToStr(miny);
  lz.Caption := IntToStr(minz);
  MainForm.RedrawPicture(nil);
  MainForm.SetFocus;
end;

procedure DoYup(b: TBlock);
begin
  with b do begin
    Inc(py, MainForm.ResXYFact);
  end
end;

procedure TAdvOptions.syTopClick(Sender: TObject);
begin
  IterateSelection(DoYup);
  lx.Caption := IntToStr(minx);
  ly.Caption := IntToStr(miny);
  lz.Caption := IntToStr(minz);
  MainForm.RedrawPicture(nil);
  MainForm.SetFocus;
end;

procedure DoZdown(b: TBlock);
begin
  with b do begin
    Dec(pz, MainForm.ResZFact);
  end
end;

procedure TAdvOptions.szBottomClick(Sender: TObject);
begin
  IterateSelection(DoZdown);
  lx.Caption := IntToStr(minx);
  ly.Caption := IntToStr(miny);
  lz.Caption := IntToStr(minz);
  MainForm.RedrawPicture(nil);
  MainForm.SetFocus;
end;

procedure DoZup(b: TBlock);
begin
  with b do begin
    Inc(pz, MainForm.ResZFact);
  end
end;

procedure TAdvOptions.szTopClick(Sender: TObject);
begin
  IterateSelection(DoZup);
  lx.Caption := IntToStr(minx);
  ly.Caption := IntToStr(miny);
  lz.Caption := IntToStr(minz);
  MainForm.RedrawPicture(nil);
  MainForm.SetFocus;
end;

procedure TAdvOptions.FormHide(Sender: TObject);
begin
  MainForm.AdvancedToolBox1.Checked := False;
end;

procedure TAdvOptions.spEditLDrawInfoClick(Sender: TObject);
begin
  if spEditLDrawInfo.Down then
    Width := GroupBox3.Left + GroupBox3.Width + 6
  else
    Width := GroupBox3.Left;
end;

procedure TAdvOptions.edNewIDChange(Sender: TObject);
var
  i                 : integer;
begin
  if not Assigned(LDrawCross) then
    InitializeLDrawCross;

  if Assigned(LDrawCross) then begin
    for i := 0 to Pred(LDrawCross.Count) do begin
      CurRec := TLDRawRecord(LDrawCross.Objects[i]);
      CurInx := i;
      if CompareText(CurRec.PartId, edNewId.Text) = 0 then begin
        spXOffsetL.AsInteger := CurRec.xOffset;
        spYOffsetL.AsInteger := CurRec.yOffset;
        spZOffsetL.AsInteger := CurRec.zOffset;
        spRotOffset.AsInteger := CurRec.RotOffset;
        GroupBox3.Enabled := True;
        Exit;
      end
    end
  end;
  CurInx := -1;
  //  GroupBox3.Enabled := False;
end;

procedure TAdvOptions.spXoffsetLChange(Sender: TObject);
begin
  if (CurInx >= 0) then begin
    CurRec := TLDrawRecord(LDrawCross.Objects[CurInx]);
    if Assigned(CurRec) then begin
      if spXOffsetL.AsInteger <> CurRec.xOffset then begin
        spXOffsetL.Font.Color := clRed;
      end
      else begin
        spXOffsetL.Font.Color := clBlack;
      end
    end
  end
end;

procedure TAdvOptions.spYoffsetLChange(Sender: TObject);
begin
  if (CurInx >= 0) then begin
    CurRec := TLDrawRecord(LDrawCross.Objects[CurInx]);
    if Assigned(CurRec) then begin
      if spYOffsetL.AsInteger <> CurRec.yOffset then begin
        spYOffsetL.Font.Color := clRed;
      end
      else begin
        spYOffsetL.Font.Color := clBlack;
      end
    end
  end
end;

procedure TAdvOptions.spZoffsetLChange(Sender: TObject);
begin
  if (CurInx >= 0) then begin
    CurRec := TLDrawRecord(LDrawCross.Objects[CurInx]);
    if Assigned(CurRec) then begin
      if spZOffsetL.AsInteger <> CurRec.zOffset then begin
        spZOffsetL.Font.Color := clRed;
      end
      else begin
        spZOffsetL.Font.Color := clBlack;
      end
    end
  end
end;

procedure TAdvOptions.spRotOffsetChange(Sender: TObject);
begin
  if (CurInx >= 0) then begin
    CurRec := TLDrawRecord(LDrawCross.Objects[CurInx]);
    if Assigned(CurRec) then begin
      if spRotOffset.AsInteger <> CurRec.RotOffset then begin
        spRotOffset.Font.Color := clRed;
      end
      else begin
        spRotOffset.Font.Color := clBlack;
      end
    end
  end
end;

procedure TAdvOptions.spUpdateLDrawClick(Sender: TObject);
begin
  if (CurInx >= 0) and Assigned(LDrawCross) then begin
    CurRec := TLDrawRecord(LDrawCross.Objects[CurInx]);
    if Assigned(CurRec) then begin
      CurRec.xOffset := spXOffsetL.AsInteger;
      CurRec.yOffset := spYOffsetL.AsInteger;
      CurRec.zOffset := spZOffsetL.AsInteger;
      CurRec.RotOffset := spRotOffset.AsInteger;
      LDrawCross[CurInx] := CrossStringFromData(CurRec);
      SaveLDrawCross;
      spXOffsetL.Font.Color := clBlack;
      spYOffsetL.Font.Color := clBlack;
      spZOffsetL.Font.Color := clBlack;
      spRotOffset.Font.Color := clBlack;
    end
  end
end;

procedure TAdvOptions.RxSpeedButton1Click(Sender: TObject);
begin
  MainForm.SpeedButton1Click(Sender);
end;

procedure TAdvOptions.spStepChange(Sender: TObject);
begin
  MainForm.RedrawPicture(nil);
end;

end.

