unit Custmize;

interface

uses
  Windows, StdCtrls, Buttons, Controls, Classes, Forms, RXCtrls
  ;

type
  TfrmCustomize = class(TForm)
    lbLeft: TListBox;
    lbRight: TListBox;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    cmbLeft: TComboBox;
    cmbRight: TComboBox;
    RxSpeedButton1: TRxSpeedButton;
    RxSpeedButton2: TRxSpeedButton;
    RxSpeedButton3: TRxSpeedButton;
    RxSpeedButton4: TRxSpeedButton;
    procedure cmbLeftChange(Sender: TObject);
    procedure cmbRightChange(Sender: TObject);
    procedure RxSpeedButton1Click(Sender: TObject);
    procedure RxSpeedButton2Click(Sender: TObject);
    procedure RxSpeedButton3Click(Sender: TObject);
    procedure RxSpeedButton4Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lbLeftDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure lbLeftMeasureItem(Control: TWinControl; Index: Integer;
      var Height: Integer);
  private
    { Private declarations }
    Pages: TStringList;
    procedure LoadPage(lb: TListBox; n: integer);
    procedure SavePage(lb: TListBox; n: integer);
    procedure MoveItem(lbFrom, lbTo: TListBox);
    procedure CopyIn;
    procedure CopyOut;
  public
    { Public declarations }
    procedure Execute;
  end;

var
  frmCustomize      : TfrmCustomize;

implementation

uses main;

{$R *.DFM}

procedure TfrmCustomize.CopyIn;
var
  i                 : integer;
begin
  for i := 0 to Pred(Pages.Count) do begin
    Pages.Objects[i].Free;
  end;
  Pages.Clear;
  for i := 0 to Pred(MainForm.Pages.Count) do begin
    Pages.AddObject(MainForm.Pages[i], TStringList.Create);
    TStrings(Pages.Objects[i]).Assign(TStrings(MainForm.Pages.Objects[i]));
  end
end;

procedure TfrmCustomize.CopyOut;
var
  i                 : integer;
begin
  for i := 0 to Pred(MainForm.Pages.Count) do begin
    MainForm.Pages.Objects[i].Free;
  end;
  MainForm.Pages.Clear;
  for i := 0 to Pred(Pages.Count) do begin
    MainForm.Pages.AddObject(Pages[i], TStringList.Create);
    TStrings(MainForm.Pages.Objects[i]).Assign(TStrings(Pages.Objects[i]));
  end
end;

procedure TfrmCustomize.LoadPage(lb: TListBox; n: integer);
begin
  if (n >= 0) and (n < Pages.Count) then begin
    if Assigned(Pages.Objects[n]) then begin
      lb.Tag := integer(MainForm.pgHeight[n]);
      lb.Items.Assign(TStrings(Pages.Objects[n]));
    end
  end
end;

procedure TfrmCustomize.SavePage(lb: TListBox; n: integer);
begin
  if (n >= 0) and (n < Pages.Count) then begin
    TStrings(Pages.Objects[n]).Assign(lb.Items);
  end
end;

procedure TfrmCustomize.Execute;
begin
  {fill up local data}
  CopyIn;
  cmbLeft.Items.Assign(MainForm.ComboBox1.Items);
  cmbLeft.ItemIndex := MainForm.CurPage + 1;
  cmbRight.Items.Assign(MainForm.ComboBox1.Items);
  cmbRight.ItemIndex := MainForm.CurPage + 1;
  LoadPage(lbLeft, MainForm.CurPage);
  LoadPage(lbRight, MainForm.CurPage);
  if ShowModal = mrOK then begin
    {store local data globally}
    CopyOut;
  end
end;

procedure TfrmCustomize.cmbLeftChange(Sender: TObject);
begin
  LoadPage(lbLeft, cmbLeft.ItemIndex - 1);
end;

procedure TfrmCustomize.cmbRightChange(Sender: TObject);
begin
  LoadPage(lbRight, cmbRight.ItemIndex - 1);
end;

procedure TfrmCustomize.MoveItem(lbFrom, lbTo: TListBox);
var
  i, j              : integer;
begin
  if lbFrom.ItemIndex >= 0 then begin
    i := lbFrom.ItemIndex;
    if lbTo.ItemIndex >= 0 then begin
      j := lbTo.ItemIndex;
      lbTo.Items.Insert(j + 1, lbFrom.Items[i]);
    end
    else begin
      lbTo.Items.Add(lbFrom.Items[i]);
    end;
    lbFrom.Items.Delete(i);
    lbFrom.ItemIndex := i;
  end
end;

procedure TfrmCustomize.RxSpeedButton1Click(Sender: TObject);
begin
  if (cmbLeft.ItemIndex > 0) and
    (cmbRight.ItemIndex > 0) and
    (cmbLeft.ItemIndex <> cmbRight.ItemIndex) then begin
    MoveItem(lbLeft, lbRight);
    SavePage(lbLeft, cmbLeft.ItemIndex - 1);
    SavePage(lbRight, cmbRight.ItemIndex - 1);
  end;
end;

procedure TfrmCustomize.RxSpeedButton2Click(Sender: TObject);
begin
  if (cmbLeft.ItemIndex > 0) and
    (cmbRight.ItemIndex > 0) and
    (cmbLeft.ItemIndex <> cmbRight.ItemIndex) then begin
    MoveItem(lbRight, lbLeft);
    SavePage(lbLeft, cmbLeft.ItemIndex - 1);
    SavePage(lbRight, cmbRight.ItemIndex - 1);
  end
end;

procedure TfrmCustomize.RxSpeedButton3Click(Sender: TObject);
var
  i                 : integer;
begin
  if lbRight.ItemIndex > 0 then begin
    i := lbRight.ItemIndex;
    lbRight.Items.Move(i, Pred(i));
    lbRight.ItemIndex := Pred(i);
    SavePage(lbRight, cmbRight.ItemIndex - 1);
  end
end;

procedure TfrmCustomize.RxSpeedButton4Click(Sender: TObject);
var
  i                 : integer;
begin
  if lbRight.ItemIndex < lbRight.Items.Count then begin
    i := lbRight.ItemIndex;
    lbRight.Items.Move(i, Succ(i));
    lbRight.ItemIndex := Succ(i);
    SavePage(lbRight, cmbRight.ItemIndex - 1);
  end
end;

procedure TfrmCustomize.FormCreate(Sender: TObject);
begin
  Pages := TStringList.Create;
  lbLeft.Controlstyle := lbLeft.Controlstyle + [csOpaque];
  lbRight.Controlstyle := lbRight.Controlstyle + [csOpaque];
end;

procedure TfrmCustomize.FormDestroy(Sender: TObject);
var
  i                 : integer;
begin
  for i := 0 to Pred(Pages.Count) do begin
    Pages.Objects[i].Free;
  end;
  Pages.Free;
end;

procedure TfrmCustomize.lbLeftDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  MainForm.ListBox1DrawItem(Control, Index, Rect, State);
end;

procedure TfrmCustomize.lbLeftMeasureItem(Control: TWinControl;
  Index: Integer; var Height: Integer);
begin
  MainForm.ListBox1MeasureItem(Control, Index, Height);
end;

end.
