unit Cutting;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, RXSpin;

type
  TDlg = class(TForm)
    edXLeft: TRxSpinEdit;
    edYBack: TRxSpinEdit;
    edZBottom: TRxSpinEdit;
    edXRight: TRxSpinEdit;
    edYFront: TRxSpinEdit;
    edZTop: TRxSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    chkAutoUpdate: TCheckBox;
    btnUpdate: TBitBtn;
    Label6: TLabel;
    RxSpinButton1: TRxSpinButton;
    RxSpinButton2: TRxSpinButton;
    RxSpinButton3: TRxSpinButton;
    BitBtn1: TBitBtn;
    procedure edXLeftChange(Sender: TObject);
    procedure btnUpdateClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure RxSpinButton1BottomClick(Sender: TObject);
    procedure RxSpinButton1TopClick(Sender: TObject);
    procedure RxSpinButton2BottomClick(Sender: TObject);
    procedure RxSpinButton2TopClick(Sender: TObject);
    procedure RxSpinButton3BottomClick(Sender: TObject);
    procedure RxSpinButton3TopClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    fXLeft, fXRight, fYBack, fYFront, fZBottom, fZTop: integer;
    fUpdate: integer;
    fNotifyChange: TNotifyEvent;
    function MaxModelZ: integer;
  public
    { Public declarations }
    procedure BeginUpdate;
    procedure EndUpdate;
    function ZDown: integer;
    function ZUp: integer;
    function ZMax: integer;
    function ZMin: integer;
    function ZLevel: integer;
    procedure UpdateIncrements;
    function BlockVisible(b: TObject): Boolean;
    procedure ResetCutting;
    property NotifyChange: TNotifyEvent read fNotifyChange write fNotifyChange;
  end;

var
  Dlg               : TDlg;

implementation

{$R *.DFM}

uses
  Main,
  LcadPc2;

var
  XLeft, XRight, YBack, YFront, ZBottom, ZTop: integer;

procedure TDlg.edXLeftChange(Sender: TObject);
begin
  if fUpdate = 0 then begin
    XLeft := edXLeft.AsInteger;
    YBack := edYBack.AsInteger;
    ZBottom := edZBottom.AsInteger;
    XRight := edXRight.AsInteger;
    YFront := edYFront.AsInteger;
    ZTop := edZTop.AsInteger;
    if chkAutoUpdate.Checked then
      btnUpdateClick(nil);
  end
end;

procedure TDlg.btnUpdateClick(Sender: TObject);
begin
  PostMessage(Main.MainForm.Handle, REDRAW_PICTURE, 0, 0);
  //  Main.MainForm.RedrawPicture(nil);
end;

procedure TDlg.FormShow(Sender: TObject);
begin
  if Assigned(fNotifyChange) then
    fNotifyChange(nil);

  UpdateIncrements;

  XLeft := fXLeft;
  YBack := fYBack;
  ZBottom := fZBottom;
  XRight := fXRight;
  YFront := fYFront;
  ZTop := fZTop;

  BeginUpdate;
  try
    edXLeft.AsInteger := XLeft;
    edYBack.AsInteger := YBack;
    edZBottom.AsInteger := ZBottom;
    edXRight.AsInteger := XRight;
    edYFront.AsInteger := YFront;
    edZTop.AsInteger := ZTop;
  finally
    EndUpdate;
  end;

  if chkAutoUpdate.Checked then
    btnUpdateClick(nil);
end;

procedure TDlg.FormHide(Sender: TObject);
begin
  if Assigned(fNotifyChange) then
    fNotifyChange(nil);

  fXLeft := XLeft;
  fYBack := YBack;
  fZBottom := ZBottom;
  fXRight := XRight;
  fYFront := YFront;
  fZTop := ZTop;

  XLeft := -19999;
  YBack := -19999;
  ZBottom := -19999;
  XRight := 19999;
  YFront := 19999;
  ZTop := 19999;

  PostMessage(Main.MainForm.Handle, REDRAW_PICTURE, 0, 0);
end;

procedure TDlg.ResetCutting;
var
  cx, cy, cz        : integer;
begin
  MainForm.FindCenter(XLeft, YBack, ZBottom, XRight, YFront, ZTop, cx, cy, cz);

  fXLeft := XLeft;
  fYBack := YBack;
  fZBottom := ZBottom;
  fXRight := XRight;
  fYFront := YFront;
  fZTop := ZTop;

  BeginUpdate;
  try
    edXLeft.AsInteger := XLeft;
    edYBack.AsInteger := YBack;
    edZBottom.AsInteger := ZBottom;
    edXRight.AsInteger := XRight;
    edYFront.AsInteger := YFront;
    edZTop.AsInteger := ZTop;
  finally
    EndUpdate;
  end;
end;

procedure TDlg.BeginUpdate;
begin
  Inc(fUpdate);
end;

procedure TDlg.EndUpdate;
begin
  Dec(fUpdate);
  if fUpdate < 0 then
    fUpdate := 0;
end;

procedure TDlg.UpdateIncrements;
begin
  edXLeft.Increment := MainForm.ResXYFact;
  edYBack.Increment := MainForm.ResXYFact;
  edZBottom.Increment := MainForm.ResZFact;
  edXRight.Increment := MainForm.ResXYFact;
  edYFront.Increment := MainForm.ResXYFact;
  edZTop.Increment := MainForm.ResZFact;
end;

procedure TDlg.RxSpinButton1BottomClick(Sender: TObject);
begin
  BeginUpdate;
  try
    edXLeft.Value := edXLeft.Value - edXLeft.Increment;
    edXRight.Value := edXRight.Value - edXRight.Increment;
  finally
    EndUpdate;
  end;
  edXLeftChange(nil);
end;

procedure TDlg.RxSpinButton1TopClick(Sender: TObject);
begin
  BeginUpdate;
  try
    edXLeft.Value := edXLeft.Value + edXLeft.Increment;
    edXRight.Value := edXRight.Value + edXRight.Increment;
  finally
    EndUpdate;
  end;
  edXLeftChange(nil);
end;

procedure TDlg.RxSpinButton2BottomClick(Sender: TObject);
begin
  BeginUpdate;
  try
    edYBack.Value := edYBack.Value - edYBack.Increment;
    edYFront.Value := edYFront.Value - edYFront.Increment;
  finally
    EndUpdate;
  end;
  edXLeftChange(nil);
end;

procedure TDlg.RxSpinButton2TopClick(Sender: TObject);
begin
  BeginUpdate;
  try
    edYBack.Value := edYBack.Value + edYBack.Increment;
    edYFront.Value := edYFront.Value + edYFront.Increment;
  finally
    EndUpdate;
  end;
  edXLeftChange(nil);
end;

procedure TDlg.RxSpinButton3BottomClick(Sender: TObject);
begin
  BeginUpdate;
  try
    edZBottom.Value := edZBottom.Value - edZBottom.Increment;
    edZTop.Value := edZTop.Value - edZTop.Increment;
  finally
    EndUpdate;
  end;
  edXLeftChange(nil);
end;

procedure TDlg.RxSpinButton3TopClick(Sender: TObject);
begin
  BeginUpdate;
  try
    edZBottom.Value := edZBottom.Value + edZBottom.Increment;
    edZTop.Value := edZTop.Value + edZTop.Increment;
  finally
    EndUpdate;
  end;
  edXLeftChange(nil);
end;

function TDlg.BlockVisible(b: TObject): Boolean;
begin
  with b as TBlock do begin
    if not Visible then
      Result := ((pz + ph) <= ZTop) and ((pz + 0) >= ZBottom)
    else
      Result := ((pz + ph) <= ZTop) and ((pz + 0) >= ZBottom) and
        ((px + pl) <= XRight) and ((px + 0) >= XLeft) and
        ((py + pb) <= YFront) and ((py + 0) >= YBack);
  end
end;

procedure TDlg.BitBtn1Click(Sender: TObject);
begin
  ResetCutting;
  btnUpdateClick(nil);
end;

procedure TDlg.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  PostMessage(Main.MainForm.Handle, REDRAW_PICTURE, 0, 0);
end;

function TDlg.MaxModelZ: integer;
var
  XLeft, YBack, ZBottom, XRight, YFront, ZTop, cx, cy, cz: integer;
begin
  MainForm.FindCenter(XLeft, YBack, ZBottom, XRight, YFront, ZTop, cx, cy, cz);
  Result := ZTop;
end;

function TDlg.ZDown: integer;
begin
  if Assigned(Self) then begin
    UpdateIncrements;
    Result := MaxModelZ;
    if fZTop <= Result then begin
      fZTop := fZTop - Trunc(edZTop.Increment);
      Result := fZTop;
    end
    else begin
      fZTop := Result;
    end;
    ZTop := fZtop;
    BeginUpdate;
    edZTop.AsInteger := fZTop;
    endUpdate;
  end
  else
    Result := 19999;
end;

function TDlg.ZUp: integer;
begin
  if Assigned(Self) then begin
    UpdateIncrements;
    fZTop := fZtop + Trunc(edZTop.Increment);
    Result := fZTop;
    ZTop := fZtop;
    BeginUpdate;
    edZTop.AsInteger := fZTop;
    EndUpdate;
  end
  else
    Result := 19999;
end;

function TDlg.ZMin: integer;
begin
  fZTop := 0;
  Result := fZTop;
  ZTop := fZtop;
  BeginUpdate;
  edZTop.AsInteger := fZTop;
  EndUpdate;
end;

function TDlg.ZMax: integer;
begin
  fZTop := 19999;
  Result := fZTop;
  ZTop := fZtop;
  BeginUpdate;
  edZTop.AsInteger := fZTop;
  EndUpdate;
end;

function TDlg.ZLevel: integer;
begin
  Result := ZBottom;
end;

initialization
  XLeft := -19999;
  YBack := -19999;
  ZBottom := -19999;
  XRight := 19999;
  YFront := 19999;
  ZTop := 19999;
end.
