(**************************************************************************
* BlockCAD, a program for building with Lego like blocks on the computer
* Copyright (C) 1998  Anders Isaksson
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
* -------------------------------------------------------------------------
* I hope You send me a copy of any important changes/additions You do.
*
* E-mail: isaksson.etuna@ebox.tninet.se
*
**************************************************************************)

unit IntList;

interface

uses
  Classes
  ;

type
  TIntegerList = class(TList)
  protected
    function GetInt(Index: integer): integer;
    procedure PutInt(Index, Value: Integer);
  public
    function Add(Item: integer): Integer;
    function First: integer;
    function IndexOf(Item: integer): Integer;
    procedure Insert(Index, Item: integer);
    function Last: integer;
    function Remove(Item: integer): Integer;
    property Items[Index: Integer]: integer read GetInt write PutInt; default;
  end;

implementation

function TIntegerList.Add(Item: integer): integer;
begin
  Result := inherited Add(Pointer(Item));
end;

function TIntegerList.First: integer;
begin
  Result := integer(inherited First);
end;

function TIntegerList.IndexOf(Item: integer): integer;
begin
  Result := inherited IndexOf(Pointer(Item));
end;

procedure TIntegerList.Insert(Index, Item: integer);
begin
  inherited Insert(Index, Pointer(Item));
end;

function TIntegerList.Last: integer;
begin
  Result := integer(inherited Last);
end;

function TIntegerList.Remove(Item: integer): integer;
begin
  Result := inherited Remove(Pointer(Item));
end;

function TIntegerList.GetInt(Index: integer): integer;
begin
  Result := integer(Get(Index));
end;

procedure TIntegerList.PutInt(Index, Value: integer);
begin
  while (Index >= Count) do begin
    Add(0);
  end;
  Put(Index, Pointer(Value));
end;

end.
