unit LdrawSupport;

interface

uses
  LCadPc2
  ;

procedure LDraw2BlockCAD(InFile: string; BlockList: TBlockList);

implementation

uses
  Classes, SysUtils, Graphics,
  Main, LCadGnrl
  ;

const
  NumRot            : array[0..7, 0..8] of single = (
    (1, 0, 0, 0, 1, 0, 0, 0, 1),
    (0, 0, -1, 0, 1, 0, 1, 0, 0),
    (-1, 0, 0, 0, 1, 0, 0, 0, -1),
    (0, 0, 1, 0, 1, 0, -1, 0, 0),
    (0.7, 0, -0.7, 0, 1, 0, 0.7, 0, 0.7),
    (-0.7, 0, -0.7, 0, 1, 0, 0.7, 0, -0.7),
    (-0.7, 0, 0.7, 0, 1, 0, -0.7, 0, -0.7),
    (0.7, 0, 0.7, 0, 1, 0, -0.7, 0, 0.7)
    );

type
  TLdrawLine = record
    PartID: string;
    x, y, z: Double;
    c: integer;
    Mat: array[0..8] of Single;
  end;

function FMax(a, b: Single): Single;
begin
  if a > b then
    Result := a
  else
    Result := b;
end;

procedure LDraw2BlockCAD(InFile: string; BlockList: TBlockList);

var
  i                 : integer;
  p                 : integer;
  DatFile           : TStringList;
  LDrawLine         : TLDrawLine;
  Ld                : TLdrawRecord;
  Block             : TBlock;
  s                 : string;
  x, y, z           : Double;
  Rot               : integer;
  tmp               : integer;

  function CheckMatrix: integer;
  var
    i               : integer;
    r, br           : integer;
    d, bd           : Single;
  begin
    bd := 99999;
    br := -1;
    for r := 0 to 3 do begin
      d := 0;
      for i := 0 to 8 do begin
        d := FMax(Abs(LDrawLine.Mat[i] - NumRot[r, i]), d);
      end;
      if d < bd then begin
        bd := d;
        br := r;
      end
    end;
    if Abs(bd) < 0.1 then
      Result := br
    else
      Result := -1;
  end;

  function Decode(s: string): Boolean;
  var
    sl              : TStringList;
  begin
    Result := False;
    sl := TStringList.Create;
    try
      sl.CommaText := s;
      if (sl.Count = 15) and (sl[0] = '1') then begin
        LDrawLine.c := StrToInt(sl[1]);
        LDrawLine.x := StrToFloat(sl[2]);
        LDrawLine.y := StrToFloat(sl[3]);
        LDrawLine.z := StrToFloat(sl[4]);
        LDrawLine.Mat[0] := StrToFloat(sl[5]);
        LDrawLine.Mat[1] := StrToFloat(sl[6]);
        LDrawLine.Mat[2] := StrToFloat(sl[7]);
        LDrawLine.Mat[3] := StrToFloat(sl[8]);
        LDrawLine.Mat[4] := StrToFloat(sl[9]);
        LDrawLine.Mat[5] := StrToFloat(sl[10]);
        LDrawLine.Mat[6] := StrToFloat(sl[11]);
        LDrawLine.Mat[7] := StrToFloat(sl[12]);
        LDrawLine.Mat[8] := StrToFloat(sl[13]);
        LDrawLine.PartID := sl[14];
        Result := True;
      end;
    finally
      sl.Free;
    end
  end;

begin
  InitializeLdrawCross;

  DatFile := TStringList.Create;

  try
    DatFile.LoadFromFile(InFile);

    for i := 0 to Pred(DatFile.Count) do begin

      if Decode(DatFile[i]) then begin

        for p := 0 to Pred(LDrawCross.Count) do begin
          Ld := TLDrawRecord(LDrawCross.Objects[p]);
          if AnsiSameText(Ld.PartID + '.dat', LDrawLine.PartID) then begin
            if Ld.t = 'G' then
              s := Format('%s %s, %s', [Ld.t, Ld.FilNam, Ld.PrtNam])
            else
              s := Format('%s %dx%dx%d', [Ld.t, Ld.b, Ld.l, Ld.h]);

            Rot := CheckMatrix;

            if Rot >= 0 then begin

              try
                Block := CreateBlockFromString(s);

                if Assigned(Block) then begin
                  Block.cl := InvColorMatch(LDrawLine.c);
                  Block.Transp := (LDrawLine.c >= 32) and (LDrawLine.c < 64);

                  x := LDrawLine.x;
                  y := LDrawLine.y - Ld.yOffset;
                  z := LDrawLine.z;

                  Block.Dir := (Rot + (4 - Ld.RotOffset)) mod 4;

                  case Rot of
                    0: begin
                        x := x + Ld.xOffset;
                        z := z - Ld.zOffset;
                      end;
                    1: begin
                        z := z + Ld.xOffset;
                        x := x + Ld.zOffset;
                        tmp := Block.pl;
                        Block.pl := Block.pb;
                        Block.pb := tmp;
                      end;
                    2: begin
                        x := x - Ld.xOffset;
                        z := z + Ld.zOffset;
                      end;
                    3: begin
                        z := z - Ld.xOffset;
                        x := x - Ld.zOffset;
                        tmp := Block.pl;
                        Block.pl := Block.pb;
                        Block.pb := tmp;
                      end;
                  end;
                  Block.px := Trunc((x / 10 * PFACTORXY - Block.pl) / 2);
                  Block.py := Trunc((-z / 10 * PFACTORXY - Block.pb) / 2);
                  Block.pz := Trunc((-y / 8 / 3 * PFACTORZ - Block.ph));

                  BlockList.Add(Block);
                  Break;
                end
              except
              end
            end                         // if Rot >= 0
          end                           // if Ld.PartID = LDrawLine.PartID
        end                             // for p := 0 to Pred(LDrawCross.Count)
      end                               // if Decode(DatFile.Items[i])
    end;                                // for i := 0 to Pred(Count)
  finally
    DatFile.Free;
  end
end;


end.
