(**************************************************************************
* BlockCAD, a program for building with Lego like blocks on the computer
* Copyright (C) 1998  Anders Isaksson
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
* -------------------------------------------------------------------------
* I hope You send me a copy of any important changes/additions You do.
*
* E-mail: isaksson.etuna@ebox.tninet.se
*
**************************************************************************)

unit Lcadpc1;
{
  Some more hardcoded blocks - window, door, wheel
}

interface

uses
  Windows, Graphics,
  LCadPc2, DrawObject
  ;

type
  TWindow = class(TBlock)
  protected
    procedure Back(DrawCanvas: TDrawObject); virtual;
    procedure LeftOuter(DrawCanvas: TDrawObject); virtual;
    procedure LeftInner(DrawCanvas: TDrawObject); virtual;
    procedure BottomOuter(DrawCanvas: TDrawObject); virtual;
    procedure FrontSlant(DrawCanvas: TDrawObject); virtual;
    procedure BottomInner(DrawCanvas: TDrawObject); virtual;
    procedure FrameTop(DrawCanvas: TDrawObject); virtual;
    procedure FrameBottom(DrawCanvas: TDrawObject); virtual;
    procedure FrameBack(DrawCanvas: TDrawObject); virtual;
    procedure RightOuter(DrawCanvas: TDrawObject); virtual;
    procedure RightInner(DrawCanvas: TDrawObject); virtual;
    procedure TopOuter(DrawCanvas: TDrawObject); virtual;
    procedure TopInner(DrawCanvas: TDrawObject); virtual;
    procedure FrontInner(DrawCanvas: TDrawObject); virtual;
    procedure FrontMiddle(DrawCanvas: TDrawObject); virtual;
    procedure FrontOuter(DrawCanvas: TDrawObject); virtual;
    procedure WindowPane(DrawCanvas: TDrawObject); virtual;
    procedure Octant0(const DrawCanvas: TDrawObject); override;
    procedure Octant1(const DrawCanvas: TDrawObject); override;
    procedure Octant2(const DrawCanvas: TDrawObject); override;
    procedure Octant3(const DrawCanvas: TDrawObject); override;
    procedure Octant4(const DrawCanvas: TDrawObject); override;
    procedure Octant5(const DrawCanvas: TDrawObject); override;
    procedure Octant6(const DrawCanvas: TDrawObject); override;
    procedure Octant7(const DrawCanvas: TDrawObject); override;
  public
    constructor CreateBlock(const xx, yy, zz, ll, bb, hh: integer;
      const cc: TColor;
      const ss: Boolean;
      const IsBasePlate: Boolean;
      const IsTransp: Boolean;
      const LDrawId: string); override;
    procedure Draw(const DrawCanvas: TDrawObject); override;
  end;

  TDoor = class(TWindow)
  protected
    procedure FrontOuter(DrawCanvas: TDrawObject); override;
    procedure FrontMiddle(DrawCanvas: TDrawObject); override;
    procedure FrontInner(DrawCanvas: TDrawObject); override;
    procedure LeftOuter(DrawCanvas: TDrawObject); override;
    procedure LeftInner(DrawCanvas: TDrawObject); override;
    procedure RightOuter(DrawCanvas: TDrawObject); override;
    procedure RightInner(DrawCanvas: TDrawObject); override;
    procedure FrontSlant(DrawCanvas: TDrawObject); override;
    procedure FrameTop(DrawCanvas: TDrawObject); override;
    procedure FrameBottom(DrawCanvas: TDrawObject); override;
    procedure FrameBack(DrawCanvas: TDrawObject); override;
    procedure WindowPane(DrawCanvas: TDrawObject); override;
  public
    constructor CreateBlock(const xx, yy, zz, ll, bb, hh: integer;
      const cc: TColor;
      const ss: Boolean;
      const IsBasePlate: Boolean;
      const IsTransp: Boolean;
      const LDrawId: string); override;
  end;

  TWheel = class(TWindow)
  protected
    procedure Octant0(const DrawCanvas: TDrawObject); override;
    procedure Octant1(const DrawCanvas: TDrawObject); override;
    procedure Octant2(const DrawCanvas: TDrawObject); override;
    procedure Octant3(const DrawCanvas: TDrawObject); override;
    procedure Octant4(const DrawCanvas: TDrawObject); override;
    procedure Octant5(const DrawCanvas: TDrawObject); override;
    procedure Octant6(const DrawCanvas: TDrawObject); override;
    procedure Octant7(const DrawCanvas: TDrawObject); override;
    procedure Circle(const DrawCanvas: TDrawObject; yy: integer);
    procedure Edge(DrawCanvas: TDrawObject);
  public
    procedure TopStuds(const DrawCanvas: TDrawObject); override;
    procedure BottomStuds(const DrawCanvas: TDrawObject); override;
    constructor CreateBlock(const xx, yy, zz, ll, bb, hh: integer;
      const cc: TColor;
      const ss: Boolean;
      const IsBasePlate: Boolean;
      const IsTransp: Boolean;
      const LDrawId: string); override;
  end;

implementation

uses
  Main;

{***********************************************************************}

constructor TWindow.CreateBlock(const xx, yy, zz, ll, bb, hh: integer;
  const cc: TColor;
  const ss: Boolean;
  const IsBasePlate: Boolean;
  const IsTransp: Boolean;
  const LDrawId: string);
begin
  inherited CreateBlock(xx, yy, zz, ll, bb, hh, cc, ss, IsBasePlate, IsTransp, LDrawID);
  t := 'W';
end;

constructor TDoor.CreateBlock(const xx, yy, zz, ll, bb, hh: integer;
  const cc: TColor;
  const ss: Boolean;
  const IsBasePlate: Boolean;
  const IsTransp: Boolean;
  const LDrawId: string);
begin
  inherited CreateBlock(xx, yy, zz, ll, bb, hh, cc, ss, IsBasePlate, IsTransp, LDrawID);
  t := 'D';
end;

constructor TWheel.CreateBlock(const xx, yy, zz, ll, bb, hh: integer;
  const cc: TColor;
  const ss: Boolean;
  const IsBasePlate: Boolean;
  const IsTransp: Boolean;
  const LDrawId: string);
begin
  inherited CreateBlock(xx, yy, zz, ll, bb, hh, cc, ss, IsBasePlate, IsTransp, LDrawID);
  t := 'O';
end;

{-TWindow-----------------------------------------------------------------}

procedure TWindow.Back(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px, py, pz), Point3D(px + 2, py, pz),
    Point3D(px + 2, py, pz + ph - 2), Point3D(px + pl - 2, py, pz + ph - 2),
      Point3D(px + pl - 2, py, pz), Point3D(px + pl, py, pz),
      Point3D(px + pl, py, pz + ph), Point3D(px, py, pz + ph)],
      Pts8, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shy;
  DrawCanvas.Polygon(Pts8);
end;

procedure TWindow.RightOuter(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px + pl, py, pz), Point3D(px + pl, py + pb, pz),
    Point3D(px + pl, py + pb, pz + 2), Point3D(px + pl, py + pb - 2, pz + 4),
      Point3D(px + pl, py + pb - 2, pz + 6), Point3D(px + pl, py + pb, pz + 6),
      Point3D(px + pl, py + pb, pz + ph), Point3D(px + pl, py, pz + ph)],
      Pts8, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shx;
  DrawCanvas.Polygon(Pts8);
end;

procedure TWindow.RightInner(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px + pl - 2, py, pz), Point3D(px + pl - 2, py + pb, pz),
    Point3D(px + pl - 2, py + pb, pz + 2), Point3D(px + pl - 2, py + pb - 2, pz + 4),
      Point3D(px + pl - 2, py + pb - 2, pz + 6), Point3D(px + pl - 2, py + pb, pz + 6),
      Point3D(px + pl - 2, py + pb, pz + ph - 2), Point3D(px + pl - 2, py, pz + ph - 2)],
      Pts8, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shx;
  DrawCanvas.Polygon(Pts8);
end;

procedure TWindow.BottomOuter(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px, py, pz), Point3D(px, py + pb, pz),
    Point3D(px + pl, py + pb, pz), Point3D(px + pl, py, pz),
      Point3D(px + pl - 2, py, pz), Point3D(px + pl - 2, py + pb - 3, pz),
      Point3D(px + 2, py + pb - 3, pz), Point3D(px + 2, py, pz)],
      Pts8, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shz;
  DrawCanvas.Polygon(Pts8);
end;

procedure TWindow.BottomInner(DrawCanvas: TDrawObject);
begin
end;

procedure TWindow.FrameTop(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px + 2, py + pb - 3, pz + 8), Point3D(px + pl - 2, py + pb - 3, pz + 8),
    Point3D(px + pl - 2, py + pb, pz + 8), Point3D(px + 2, py + pb, pz + 8)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shz;
  DrawCanvas.Polygon(Pts4);
end;

procedure TWindow.FrameBottom(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px, py + pb - 3, pz + 6), Point3D(px + pl, py + pb - 3, pz + 6),
    Point3D(px + pl, py + pb, pz + 6), Point3D(px, py + pb, pz + 6)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shz;
  DrawCanvas.Polygon(Pts4);
end;

procedure TWindow.FrameBack(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px + 2, py + pb - 3, pz + 8), Point3D(px + pl - 2, py + pb - 3, pz + 8),
    Point3D(px + pl - 2, py + pb - 3, pz), Point3D(px + 2, py + pb - 3, pz)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shy;
  DrawCanvas.Polygon(Pts4);
end;

procedure TWindow.LeftOuter(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px, py, pz), Point3D(px, py + pb, pz),
    Point3D(px, py + pb, pz + 2), Point3D(px, py + pb - 2, pz + 4),
      Point3D(px, py + pb - 2, pz + 6), Point3D(px, py + pb, pz + 6),
      Point3D(px, py + pb, pz + ph), Point3D(px, py, pz + ph)],
      Pts8, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shx;
  DrawCanvas.Polygon(Pts8);
end;

procedure TWindow.LeftInner(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px + 2, py, pz), Point3D(px + 2, py + pb, pz),
    Point3D(px + 2, py + pb, pz + 2), Point3D(px + 2, py + pb - 2, pz + 4),
      Point3D(px + 2, py + pb - 2, pz + 6), Point3D(px + 2, py + pb, pz + 6),
      Point3D(px + 2, py + pb, pz + ph - 2), Point3D(px + 2, py, pz + ph - 2)],
      Pts8, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shx;
  DrawCanvas.Polygon(Pts8);
end;

procedure TWindow.FrontInner(DrawCanvas: TDrawObject);
begin
end;

procedure TWindow.FrontMiddle(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px, py + pb - 2, pz + 4), Point3D(px + pl, py + pb - 2, pz + 4),
    Point3D(px + pl, py + pb - 2, pz + 6), Point3D(px, py + pb - 2, pz + 6)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shy;
  DrawCanvas.Polygon(Pts4);
end;

procedure TWindow.FrontOuter(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px, py + pb, pz + 6), Point3D(px + 2, py + pb, pz + 6),
    Point3D(px + 2, py + pb, pz + ph - 2), Point3D(px + pl - 2, py + pb, pz + ph - 2),
      Point3D(px + pl - 2, py + pb, pz + 6), Point3D(px + pl, py + pb, pz + 6),
      Point3D(px + pl, py + pb, pz + ph), Point3D(px, py + pb, pz + ph)],
      Pts8, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shy;
  DrawCanvas.Polygon(Pts8);
  Transform([Point3D(px, py + pb, pz), Point3D(px, py + pb, pz + 2),
    Point3D(px + pl, py + pb, pz + 2), Point3D(px + pl, py + pb, pz)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shy;
  DrawCanvas.Polygon(Pts4);
  Transform([Point3D(px + 2, py + pb, pz + 8), Point3D(px + pl - 2, py + pb, pz + 8),
    Point3D(px + pl - 2, py + pb, pz + 6), Point3D(px + 2, py + pb, pz + 6)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shy;
  DrawCanvas.Polygon(Pts4);
end;

procedure TWindow.FrontSlant(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px, py + pb, pz + 2), Point3D(px + pl, py + pb, pz + 2),
    Point3D(px + pl, py + pb - 2, pz + 4), Point3D(px, py + pb - 2, pz + 4)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shsy;
  DrawCanvas.Polygon(Pts4);
end;

procedure TWindow.TopOuter(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px, py, pz + ph), Point3D(px + pl, py, pz + ph),
    Point3D(px + pl, py + pb, pz + ph), Point3D(px, py + pb, pz + ph)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shz;
  DrawCanvas.Polygon(Pts4);
end;

procedure TWindow.TopInner(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px + 2, py, pz + ph - 2), Point3D(px + pl - 2, py, pz + ph - 2),
    Point3D(px + pl - 2, py + pb, pz + ph - 2), Point3D(px + 2, py + pb, pz + ph - 2)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shz;
  DrawCanvas.Polygon(Pts4);
end;

procedure TWindow.WindowPane(DrawCanvas: TDrawObject);
var
  bs                : TBrushStyle;
  bc                : TColor;
  pc                : TColor;
  //  cl: TColor;
begin
  Transform([Point3D(px + 2, py + pb - 2, pz + 8), Point3D(px + pl - 2, py + pb - 2, pz + 8),
    Point3D(px + pl - 2, py + pb - 2, pz + ph - 2), Point3D(px + 2, py + pb - 2, pz + ph - 2)],
      Pts4, Dir);
  bs := DrawCanvas.Canvas.Brush.Style;
  bc := DrawCanvas.Canvas.Brush.Color;
  pc := DrawCanvas.Canvas.Pen.Color;
  DrawCanvas.Canvas.Brush.Style := bsDiagCross; { Transparent piece }
  DrawCanvas.Canvas.Brush.Color := shtr;
  DrawCanvas.Canvas.Pen.Color := shtr;
  DrawCanvas.Polygon(Pts4);
  DrawCanvas.Canvas.Pen.Color := pc;
  DrawCanvas.Canvas.Brush.Color := bc;
  DrawCanvas.Canvas.Brush.Style := bs;
end;

procedure TWindow.Octant0(const DrawCanvas: TDrawObject);
begin
  RightInner(DrawCanvas);
  BottomInner(DrawCanvas);
  FrontSlant(DrawCanvas);
  FrontMiddle(DrawCanvas);
  FrameTop(DrawCanvas);
  WindowPane(DrawCanvas);
  FrontInner(DrawCanvas);
  FrontOuter(DrawCanvas);
  LeftOuter(DrawCanvas);
  TopOuter(DrawCanvas);
end;

procedure TWindow.Octant1(const DrawCanvas: TDrawObject);
begin
  FrontOuter(DrawCanvas);
  RightInner(DrawCanvas);
  FrontInner(DrawCanvas);
  FrameTop(DrawCanvas);
  WindowPane(DrawCanvas);
  FrameBack(DrawCanvas);
  BottomInner(DrawCanvas);
  LeftOuter(DrawCanvas);
  Back(DrawCanvas);
  TopOuter(DrawCanvas);
end;

procedure TWindow.Octant2(const DrawCanvas: TDrawObject);
begin
  FrontOuter(DrawCanvas);
  LeftInner(DrawCanvas);
  FrontInner(DrawCanvas);
  FrameTop(DrawCanvas);
  WindowPane(DrawCanvas);
  FrameBack(DrawCanvas);
  BottomInner(DrawCanvas);
  RightOuter(DrawCanvas);
  Back(DrawCanvas);
  TopOuter(DrawCanvas);
end;

procedure TWindow.Octant3(const DrawCanvas: TDrawObject);
begin
  LeftInner(DrawCanvas);
  BottomInner(DrawCanvas);
  FrontInner(DrawCanvas);
  FrontSlant(DrawCanvas);
  FrontMiddle(DrawCanvas);
  FrameTop(DrawCanvas);
  WindowPane(DrawCanvas);
  RightOuter(DrawCanvas);
  FrontOuter(DrawCanvas);
  TopOuter(DrawCanvas);
end;

procedure TWindow.Octant4(const DrawCanvas: TDrawObject);
begin
  TopInner(DrawCanvas);
  RightInner(DrawCanvas);
  WindowPane(DrawCanvas);
  BottomInner(DrawCanvas);
  FrameBottom(DrawCanvas);
  FrontInner(DrawCanvas);
  FrontMiddle(DrawCanvas);
  FrontSlant(DrawCanvas);
  FrontOuter(DrawCanvas);
  LeftOuter(DrawCanvas);
  BottomOuter(DrawCanvas);
end;

procedure TWindow.Octant5(const DrawCanvas: TDrawObject);
begin
  FrontOuter(DrawCanvas);
  TopInner(DrawCanvas);
  FrameBottom(DrawCanvas);
  RightInner(DrawCanvas);
  WindowPane(DrawCanvas);
  FrontInner(DrawCanvas);
  FrameBack(DrawCanvas);
  BottomInner(DrawCanvas);
  Back(DrawCanvas);
  LeftOuter(DrawCanvas);
  BottomOuter(DrawCanvas);
end;

procedure TWindow.Octant6(const DrawCanvas: TDrawObject);
begin
  FrontOuter(DrawCanvas);
  TopInner(DrawCanvas);
  FrameBottom(DrawCanvas);
  LeftInner(DrawCanvas);
  WindowPane(DrawCanvas);
  FrontInner(DrawCanvas);
  FrameBack(DrawCanvas);
  BottomInner(DrawCanvas);
  Back(DrawCanvas);
  RightOuter(DrawCanvas);
  BottomOuter(DrawCanvas);
end;

procedure TWindow.Octant7(const DrawCanvas: TDrawObject);
begin
  TopInner(DrawCanvas);
  LeftInner(DrawCanvas);
  WindowPane(DrawCanvas);
  BottomInner(DrawCanvas);
  FrameBottom(DrawCanvas);
  FrontInner(DrawCanvas);
  FrontMiddle(DrawCanvas);
  FrontSlant(DrawCanvas);
  FrontOuter(DrawCanvas);
  RightOuter(DrawCanvas);
  BottomOuter(DrawCanvas);
end;

procedure TWindow.Draw(const DrawCanvas: TDrawObject);
var
  xOct              : integer;
  c                 : TColor;
  xpx, xpy, xpl, xpb: integer;
begin
  if (pl = 0) then Exit;

  xpx := px;
  xpy := py;
  xpl := pl;
  xpb := pb;

  if not Selected then begin
    DrawCanvas.Canvas.Brush.Color := cl;
    DrawCanvas.Canvas.Pen.Color := EdgeColor(cl);
  end
  else begin
    DrawCanvas.Canvas.Brush.Color := cl;
    if cl = RedColor then
      DrawCanvas.Canvas.Pen.Color := clWhite
    else
      DrawCanvas.Canvas.Pen.Color := RedColor;
  end;

  DoShade := MainForm.miShading.Checked;

  if (bp and MainForm.WireBase and (Octant > 3)) or
    (MainForm.WireFrame) then begin
    DrawCanvas.Canvas.Brush.Style := bsClear; { Draw as Wire }
    DrawCanvas.Canvas.Pen.Color := cl;  {}
    DoShade := False;
  end
  else if (cl = clAqua) or (Transp) then begin
    DrawCanvas.Canvas.Brush.Style := bsDiagCross; { Transparent piece }
  end;

  shx := DrawCanvas.Canvas.Brush.Color;
  shy := shx;
  shz := shx;
  shsx := shx;
  shsy := shx;
  shtr := clWhite;

  case Dir of
    0: begin
        if (DoShade) then begin
          ShadeEdge(DrawCanvas.Canvas.Brush.Color, DrawCanvas.Canvas.Pen.Color, EdgeShade, c);
          if not Selected then
            DrawCanvas.Canvas.Pen.Color := c;
          Shade(DrawCanvas.Canvas.Brush.Color, lfy, shx);
          Shade(DrawCanvas.Canvas.Brush.Color, lfx, shy);
          Shade(DrawCanvas.Canvas.Brush.Color, lfz, shz);
          Shade(DrawCanvas.Canvas.Brush.Color, lfsx, shsy);
          Shade(DrawCanvas.Canvas.Brush.Color, lfsy, shsx);
          Shade(shtr, lfy, shtr);
        end
      end;
    1: begin
        px := -(xpy + xpb);
        py := xpx;
        pl := xpb;
        pb := xpl;
        if (DoShade) then begin
          ShadeEdge(DrawCanvas.Canvas.Brush.Color, DrawCanvas.Canvas.Pen.Color, EdgeShade, c);
          if not Selected then
            DrawCanvas.Canvas.Pen.Color := c;
          Shade(DrawCanvas.Canvas.Brush.Color, lfx, shx);
          Shade(DrawCanvas.Canvas.Brush.Color, lfy, shy);
          Shade(DrawCanvas.Canvas.Brush.Color, lfz, shz);
          Shade(DrawCanvas.Canvas.Brush.Color, lfsx, shsx);
          Shade(DrawCanvas.Canvas.Brush.Color, lfsy, shsy);
          Shade(shtr, lfx, shtr);
        end
      end;
    2: begin
        px := (-xpx - xpl);
        py := (-xpy - xpb);
        if (DoShade) then begin
          ShadeEdge(DrawCanvas.Canvas.Brush.Color, DrawCanvas.Canvas.Pen.Color, EdgeShade, c);
          if not Selected then
            DrawCanvas.Canvas.Pen.Color := c;
          Shade(DrawCanvas.Canvas.Brush.Color, lfy, shx);
          Shade(DrawCanvas.Canvas.Brush.Color, lfx, shy);
          Shade(DrawCanvas.Canvas.Brush.Color, lfz, shz);
          Shade(DrawCanvas.Canvas.Brush.Color, lfsx, shsy);
          Shade(DrawCanvas.Canvas.Brush.Color, lfsy, shsx);
          Shade(shtr, lfy, shtr);
        end
      end;
    3: begin
        px := xpy;
        py := (-xpx - xpl);
        pl := xpb;
        pb := xpl;
        if (DoShade) then begin
          ShadeEdge(DrawCanvas.Canvas.Brush.Color, DrawCanvas.Canvas.Pen.Color, EdgeShade, c);
          if not Selected then
            DrawCanvas.Canvas.Pen.Color := c;
          Shade(DrawCanvas.Canvas.Brush.Color, lfx, shx);
          Shade(DrawCanvas.Canvas.Brush.Color, lfy, shy);
          Shade(DrawCanvas.Canvas.Brush.Color, lfz, shz);
          Shade(DrawCanvas.Canvas.Brush.Color, lfsx, shsx);
          Shade(DrawCanvas.Canvas.Brush.Color, lfsy, shsy);
          Shade(shtr, lfx, shtr);
        end
      end;
  end;

  if Octant < 4 then
    xOct := (Octant + Dir) and 3
  else
    xOct := ((Octant - 4 + Dir) and 3) + 4;

  case xOct of
    0: Octant0(DrawCanvas);
    1: Octant1(DrawCanvas);
    2: Octant2(DrawCanvas);
    3: Octant3(DrawCanvas);
    4: Octant4(DrawCanvas);
    5: Octant5(DrawCanvas);
    6: Octant6(DrawCanvas);
    7: Octant7(DrawCanvas);
  end;

  px := xpx;
  py := xpy;
  pl := xpl;
  pb := xpb;

  if (st and not MainForm.btnStuds.Down) then
    if (Octant < 4) then
      TopStuds(DrawCanvas)
    else
      BottomStuds(DrawCanvas);

  DrawCanvas.Canvas.Brush.Color := clSilver;
  DrawCanvas.Canvas.Brush.Style := bsSolid;
  DrawCanvas.Canvas.Pen.Color := BlackColor;
end;

{-TDoor-------------------------------------------------------------------}

procedure TDoor.FrontOuter(DrawCanvas: TDrawObject);
var
  m                 : integer;
begin
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shy;

  m := pl div 2;

  Transform([Point3D(px, py + pb, pz), Point3D(px, py + pb, pz + ph),
    Point3D(px + pl, py + pb, pz + ph), Point3D(px + pl, py + pb, pz),
      Point3D(px + pl - 2, py + pb, pz), Point3D(px + pl - 2, py + pb, pz + ph - 2),
      Point3D(px + 2, py + pb, pz + ph - 2), Point3D(px + 2, py + pb, pz)],
      Pts8, Dir);
  DrawCanvas.Polygon(Pts8);
  Transform([Point3D(px + 2, py + pb, pz), Point3D(px + 2, py + pb, pz + 2),
    Point3D(px + pl - 2, py + pb, pz + 2), Point3D(px + pl - 2, py + pb, pz)],
      Pts4, Dir);
  DrawCanvas.Polygon(Pts4);
  Transform([Point3D(px + 2, py + pb - 1, pz), Point3D(px + 2, py + pb - 1, pz + ph - 2),
    Point3D(px + m, py + pb - 1, pz + ph - 2),
      Point3D(px + m, py + pb - 1, pz + ph - 2 - 6),
      Point3D(px + m - 12, py + pb - 1, pz + ph - 2 - 16),
      Point3D(px + m, py + pb - 1, pz + ph - 2 - 26),
      Point3D(px + m, py + pb - 1, pz)],
      Pts7, Dir);
  DrawCanvas.Polygon(Pts7);
  Transform([Point3D(px + pl - 2, py + pb - 1, pz), Point3D(px + pl - 2, py + pb - 1, pz + ph - 2),
    Point3D(px + m, py + pb - 1, pz + ph - 2),
      Point3D(px + m, py + pb - 1, pz + ph - 2 - 6),
      Point3D(px + m + 12, py + pb - 1, pz + ph - 2 - 16),
      Point3D(px + m, py + pb - 1, pz + ph - 2 - 26),
      Point3D(px + m, py + pb - 1, pz)],
      Pts7, Dir);
  DrawCanvas.Polygon(Pts7);
end;

procedure TDoor.FrontMiddle(DrawCanvas: TDrawObject);
begin
  {empty}
end;

procedure TDoor.FrontInner(DrawCanvas: TDrawObject);
var
  m                 : integer;
begin
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shz;

  m := pl div 2;
  Transform([Point3D(px + 2, py + pb - 3, pz), Point3D(px + 2, py + pb - 3, pz + ph - 2),
    Point3D(px + m, py + pb - 3, pz + ph - 2),
      Point3D(px + m, py + pb - 3, pz + ph - 2 - 6),
      Point3D(px + m - 12, py + pb - 3, pz + ph - 2 - 16),
      Point3D(px + m, py + pb - 3, pz + ph - 2 - 26),
      Point3D(px + m, py + pb - 3, pz)],
      Pts7, Dir);
  DrawCanvas.Polygon(Pts7);
  Transform([Point3D(px + pl - 2, py + pb - 3, pz), Point3D(px + pl - 2, py + pb - 3, pz + ph - 2),
    Point3D(px + m, py + pb - 3, pz + ph - 2),
      Point3D(px + m, py + pb - 3, pz + ph - 2 - 6),
      Point3D(px + m + 12, py + pb - 3, pz + ph - 2 - 16),
      Point3D(px + m, py + pb - 3, pz + ph - 2 - 26),
      Point3D(px + m, py + pb - 3, pz)],
      Pts7, Dir);
  DrawCanvas.Polygon(Pts7);
end;

procedure TDoor.LeftOuter(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px, py, pz), Point3D(px, py + pb, pz),
    Point3D(px, py + pb, pz + ph), Point3D(px, py, pz + ph)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shx;
  DrawCanvas.Polygon(Pts4);
end;

procedure TDoor.LeftInner(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px + 2, py, pz), Point3D(px + 2, py + pb, pz),
    Point3D(px + 2, py + pb, pz + ph - 2), Point3D(px + 2, py, pz + ph - 2)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shx;
  DrawCanvas.Polygon(Pts4);
end;

procedure TDoor.RightOuter(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px + pl, py, pz), Point3D(px + pl, py + pb, pz),
    Point3D(px + pl, py + pb, pz + ph), Point3D(px + pl, py, pz + ph)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shx;
  DrawCanvas.Polygon(Pts4);
end;

procedure TDoor.RightInner(DrawCanvas: TDrawObject);
begin
  Transform([Point3D(px + pl - 2, py, pz), Point3D(px + pl - 2, py + pb, pz),
    Point3D(px + pl - 2, py + pb, pz + ph - 2), Point3D(px + pl - 2, py, pz + ph - 2)],
      Pts4, Dir);
  if DoShade then
    DrawCanvas.Canvas.Brush.Color := shx;
  DrawCanvas.Polygon(Pts4);
end;

procedure TDoor.FrontSlant(DrawCanvas: TDrawObject);
begin
  {empty}
end;

procedure TDoor.FrameTop(DrawCanvas: TDrawObject);
begin
  {empty}
end;

procedure TDoor.FrameBottom(DrawCanvas: TDrawObject);
begin
  {empty}
end;

procedure TDoor.FrameBack(DrawCanvas: TDrawObject);
begin
  {empty}
end;

procedure TDoor.WindowPane(DrawCanvas: TDrawObject);
begin
  {empty}
end;

{-TWheel-------------------------------------------------------------------}

const
  F1                = 0.65;
  F2                = 1.0;
  SPOKES            = 12;

  K1                = 28;

procedure TWheel.TopStuds(const DrawCanvas: TDrawObject);
begin
  {empty}
end;

procedure TWheel.BottomStuds(const DrawCanvas: TDrawObject);
begin
  {empty}
end;

procedure TWheel.Circle(const DrawCanvas: TDrawObject; yy: integer);
var
  a1, i             : integer;
  c1, s1, c2, s2    : TMyFloat;
  mx, mz            : integer;
  clb, clp          : TColor;
  ps                : integer;
begin
  a1 := 0;
  c1 := 1.0 / 2.0;
  s1 := 0.0;
  mx := px + pl div 2;
  mz := pz + K1;
  ps := pl * 3 div 4;

  clb := DrawCanvas.Canvas.Brush.Color;
  clp := DrawCanvas.Canvas.Pen.Color;

  if (bp and MainForm.WireBase and (Octant > 3)) or
    (MainForm.WireFrame) then begin
    DrawCanvas.Canvas.Brush.Style := bsClear; { Draw as Wire }
    DoShade := False;
    DrawCanvas.Canvas.Pen.Color := cl;  {}
  end
  else if (cl = clAqua) or (Transp) then
    DrawCanvas.Canvas.Brush.Style := bsDiagCross; { Transparent piece }

  for i := 1 to SPOKES do begin
    c2 := c1;
    s2 := s1;
    Inc(a1, 360 div SPOKES);
    c1 := Cos(a1 * PI / 180) / 2.0;
    s1 := Sin(a1 * PI / 180) / 2.0;
    Transform([Point3D(mx, yy, mz),
      Point3D(Round(mx + ps * c1 * F1), yy, Round(mz + ps * s1 * F1)),
        Point3D(Round(mx + ps * c2 * F1), yy, Round(mz + ps * s2 * F1))],
        Pts3, Dir);

    DrawCanvas.Polygon(Pts3);

    DrawCanvas.Canvas.Brush.Color := clGray; // - $202020;
    DrawCanvas.Canvas.Pen.Color := clGray; // - $202020;

    Transform([Point3D(Round(mx + ps * c1 * F1), yy, Round(mz + ps * s1 * F1)),
      Point3D(Round(mx + ps * c1 * F2), yy, Round(mz + ps * s1 * F2)),
        Point3D(Round(mx + ps * c2 * F2), yy, Round(mz + ps * s2 * F2)),
        Point3D(Round(mx + ps * c2 * F1), yy, Round(mz + ps * s2 * F1))],
        Pts4, Dir);
    DrawCanvas.Polygon(Pts4);

    DrawCanvas.Canvas.Brush.Color := clb;
    DrawCanvas.Canvas.Pen.Color := clp;
  end;
end;

procedure TWheel.Edge(DrawCanvas: TDrawObject);
var
  a1, i             : integer;
  c1, s1, c2, s2    : TMyFloat;
  mx, mz            : integer;
  clb, clp          : TColor;
  ps                : integer;
begin
  a1 := 0;
  c1 := 1.0 / 2.0;
  s1 := 0.0;
  mx := px + pl div 2;
  mz := pz + K1;
  ps := pl * 3 div 4;
  clb := DrawCanvas.Canvas.Brush.Color;
  clp := DrawCanvas.Canvas.Pen.Color;
  DrawCanvas.Canvas.Brush.Color := clGray - $202020;
  ;
  DrawCanvas.Canvas.Pen.Color := clGray - $202020;
  ;
  if (bp and MainForm.WireBase and (Octant > 3)) or
    (MainForm.WireFrame) then begin
    DrawCanvas.Canvas.Brush.Style := bsClear; { Draw as Wire }
    DoShade := False;
    DrawCanvas.Canvas.Pen.Color := cl;  {}
  end
  else if (cl = clAqua) or (Transp) then
    DrawCanvas.Canvas.Brush.Style := bsDiagCross; { Transparent piece }

  for i := 1 to SPOKES do begin
    c2 := c1;
    s2 := s1;
    Inc(a1, 360 div SPOKES);
    c1 := Cos(a1 * PI / 180) / 2.0;
    s1 := Sin(a1 * PI / 180) / 2.0;
    Transform([Point3D(Round(mx + ps * c1 * F2), py + 1, Round(mz + ps * s1 * F2)),
      Point3D(Round(mx + ps * c2 * F2), py + 1, Round(mz + ps * s2 * F2)),
        Point3D(Round(mx + ps * c2 * F2), py + pb - 1, Round(mz + ps * s2 * F2)),
        Point3D(Round(mx + ps * c1 * F2), py + pb - 1, Round(mz + ps * s1 * F2))],
        Pts4, Dir);
    DrawCanvas.Polygon(Pts4);
  end;
  DrawCanvas.Canvas.Brush.Color := clb;
  DrawCanvas.Canvas.Pen.Color := clp;
end;

procedure TWheel.Octant0(const DrawCanvas: TDrawObject);
begin
  Circle(DrawCanvas, py + 2);
  Edge(DrawCanvas);
  Circle(DrawCanvas, py + pb - 2);
end;

procedure TWheel.Octant1(const DrawCanvas: TDrawObject);
begin
  Circle(DrawCanvas, py + pb - 2);
  Edge(DrawCanvas);
  Circle(DrawCanvas, py + 2);
end;

procedure TWheel.Octant2(const DrawCanvas: TDrawObject);
begin
  Circle(DrawCanvas, py + pb - 2);
  Edge(DrawCanvas);
  Circle(DrawCanvas, py + 2);
end;

procedure TWheel.Octant3(const DrawCanvas: TDrawObject);
begin
  Circle(DrawCanvas, py + 2);
  Edge(DrawCanvas);
  Circle(DrawCanvas, py + pb - 2);
end;

procedure TWheel.Octant4(const DrawCanvas: TDrawObject);
begin
  Circle(DrawCanvas, py + 2);
  Edge(DrawCanvas);
  Circle(DrawCanvas, py + pb - 2);
end;

procedure TWheel.Octant5(const DrawCanvas: TDrawObject);
begin
  Circle(DrawCanvas, py + pb - 2);
  Edge(DrawCanvas);
  Circle(DrawCanvas, py + 2);
end;

procedure TWheel.Octant6(const DrawCanvas: TDrawObject);
begin
  Circle(DrawCanvas, py + pb - 2);
  Edge(DrawCanvas);
  Circle(DrawCanvas, py + 2);
end;

procedure TWheel.Octant7(const DrawCanvas: TDrawObject);
begin
  Circle(DrawCanvas, py + 2);
  Edge(DrawCanvas);
  Circle(DrawCanvas, py + pb - 2);
end;

end.
